
# q-expansion of newform 350.4.c.j, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 350
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [-1, 0]], [101, [1, 0]]]
aps_data = [[0, -2], [0, 3], [0, 0], [0, -7], [-17, 0], [0, 81], [0, -91], [-102, 0], [0, 90], [129, 0], [116, 0], [0, 314], [-124, 0], [0, 434], [0, 497], [0, 584], [332, 0], [220, 0], [0, 384], [-664, 0], [0, -230], [-361, 0], [0, -1172], [-40, 0], [0, -175], [270, 0], [0, -565], [0, -1206], [1419, 0], [0, 778], [0, 316], [-1750, 0], [0, 48], [38, 0], [3582, 0], [-1927, 0], [0, -2510], [0, 1870], [0, -2019], [0, -1253], [-492, 0], [-4448, 0], [3999, 0], [0, -4880], [0, 4712], [-2756, 0], [697, 0], [0, 707], [0, -559], [3836, 0], [0, 1368], [3803, 0], [5650, 0], [-1650, 0], [0, 5810], [0, 2758], [-186, 0], [-6828, 0], [0, -8214], [6707, 0], [0, 5497], [0, -313], [0, 5479], [4422, 0], [0, -275], [0, -4106], [8260, 0], [0, 9946], [0, -8798], [1838, 0], [0, -2265], [-4136, 0], [0, -10031], [0, -2792], [-9948, 0], [0, -2700], [631, 0], [0, -13139], [-887, 0], [5190, 0], [5712, 0], [-11155, 0], [8499, 0], [0, -5102], [-15694, 0], [0, -9142], [17145, 0], [0, -10184], [-9152, 0], [0, 1084], [0, 19283], [5118, 0], [0, 2918], [-18627, 0], [-12843, 0], [0, 18837], [7742, 0], [614, 0], [0, -17836], [23463, 0], [0, -1860], [0, 10288], [0, 2356], [15318, 0], [-13164, 0], [0, 8959], [0, 16952], [0, -10071], [2829, 0], [-7662, 0], [0, 907], [0, 29646], [0, 11006], [-21098, 0], [21707, 0], [11090, 0], [0, 995], [0, -20792], [0, 10498], [6749, 0], [12400, 0], [0, -16024], [0, 18735], [0, 12844], [25996, 0], [25435, 0], [-3823, 0], [-21090, 0], [0, -7992], [0, 36965], [3305, 0], [0, 2208], [-11551, 0], [0, 9688], [-7014, 0], [6278, 0], [0, -21351], [0, -25967], [0, -8595], [13671, 0], [-11986, 0], [-12951, 0], [0, 12788], [0, 24662], [-4124, 0], [33154, 0], [0, -12050], [0, 15270], [12564, 0], [0, -12976], [0, -15634], [-8896, 0], [0, 33456], [0, 8288], [0, 15398], [22656, 0], [-9149, 0], [27272, 0], [0, 44177], [-2236, 0], [0, -17464], [0, 18336], [0, -46802], [45024, 0], [0, 12786], [0, -33539], [-18200, 0], [0, -32599]]
